/*
 *	filename: main.cpp
 *	author  : Mark Six
 *	copyright Oct 2005, all rights reserved.
 *
 *	Currently the software assumes you have a Spartan-3 Device and optionally a xcf-prom.
 *      in the same JTAG-chain.
 *      This software is tested on the xc3200 Xilinx Development Board.
 */

#include <iostream>
#include <stdint.h>
#include "jtag.h"

using namespace std ;



bool IDCODE_PROM[] = { 0, 1, 1, 1, 1, 1, 1, 1 } ;
bool BYPASS_PROM[] = { 1, 1, 1, 1, 1, 1, 1, 1 } ;
bool IDCODE[] =      { 1, 0, 0, 1, 0, 0 } ;
bool CFG_IN[] =      { 1, 0, 1, 0, 0, 0 } ;
bool JSTART[] =      { 0, 0, 1, 1, 0, 0 } ;
bool JSHUTDOWN[] =   { 1, 0, 1, 1, 0, 0 } ;
bool USER1[]  =      { 0, 1, 0, 0, 0, 0 } ;
bool USER2[]  =      { 1, 1, 0, 0, 0, 0 } ;
 
#define XCF02S   0x05045093		// 1100100100001010001000001010000
#define XC3S50   0x0140C093		// 1100100100001010001000001010000
#define XC3S200  0x01414093
#define XC3S400  0x0141C093
#define XC3S1000 0x01428093
#define XC3S1500 0x01434093
#define XC3S2000 0x01440093
#define XC3S4000 0x01448093
#define XC3S5000 0x01450093

#define ID_LEN		32	

const char *id_string[] =
{
	"XS3S50", "XS3S200", "XS3S400", "XS3S1000", "XS3S1500", "XS3S2000", "XS3S4000", "XS3S5000",
	"XCF02S" 
} ;

const uint32_t id_code[] =
{
	XC3S50, XC3S200, XC3S400, XC3S1000, XC3S1500, XC3S2000, XC3S4000, XC3S5000,
	XCF02S
} ;

#define MAX_IDS ( sizeof( id_code ) / sizeof( int ) ) 

int getDevice( bool *id ) {
	unsigned int j, dev_id ;
	for ( j = 0, dev_id = 0 ; j < ID_LEN ; j++ ) {
		dev_id |= id[ j ] << j ;
//		cout << id[j] ;
	}
//	cout << endl ;
		
	for ( j = 0 ; j < MAX_IDS ; j++ ) {
		if ( id_code[j] == dev_id ) {
			cout << "Found Device : " << id_string[j] << "\n";
			return dev_id ;
		}
	}

	cout << "Unknown ID: " << hex << dev_id << "\n" ;

	return 0 ;
}

int main( int nargs, char *args[] )
{
	bool id[ ID_LEN], ones[ ID_LEN ], dummy[ 32 ] ;
	int device0, device1 ;
	bool prom_present = true ;

	for ( int i = 0 ; i < ID_LEN ; i++ )
		ones[i] = 1 ;

	CJTAG dev ;

	if ( !dev.open( "/dev/parport0") ) {
		cerr << "JTAG port could not be opened" << endl ;
		return -1  ;
	}
/* ...Get device(s)... */
	dev.selectIR() ;
	dev.setIR( IDCODE_PROM, 8 ) ;
	dev.setIR( IDCODE, 5 ) ;
	dev.exitIR( IDCODE[5] ) ;

	dev.selectDR() ;
	dev.setDR( ones, id, ID_LEN ) ;

	device0 = getDevice( id ) ;

	if ( !device0 ) 
		goto exit ;

	dev.setDR( ones, id, ID_LEN-1 ) ;
	id[ 31 ] = dev.exitDR( 1 ) ;

	device1 = getDevice( id ) ;
	if ( !device1 ) {
		cerr <<  "Second device is unknown (I will try to continue)" << endl  ;
		prom_present = false ;
	}

/* ....Setup for configuration..... */
	dev.selectIR();
	dev.setIR( BYPASS_PROM, 8 ) ;
	dev.setIR( JSHUTDOWN, 5 ) ;
	dev.exitIR( JSHUTDOWN[5] ) ;

	dev.selectRunTestIdle() ;
	for ( int i = 0 ; i < 20 ; i++ )		/* Shutdown sequence */
		dev.execute( 0, 0 ) ;

	dev.selectIR() ;
	if ( prom_present ) dev.setIR( BYPASS_PROM, 8 ) ;
	dev.setIR( CFG_IN, 5 ) ;
	dev.exitIR( CFG_IN[5] ) ;

/* ....Send bit file.....*/
	FILE *bit_file ;
	if ( nargs < 2 ) {
		cout << "reading from stdin, since no filename is given" << endl ;
		bit_file = stdin ;
	} else {
		bit_file = fopen( args[1], "r" ) ;
		if ( bit_file == NULL ) {
			cerr << "Could not read bit file" << endl ;
			cerr << strerror( errno ) << endl ;
			goto exit ;
		}
	}

	int size  ;
	bool frame[ 8 ] ;
	unsigned int cur, next ;
	size = 0 ;
	
	dev.selectDR() ;
	fread( &cur, 1, 1, bit_file ) ;

	int prog, prev_prog, total ;
	fseek( bit_file, 0, SEEK_END ) ;
	total = ftell( bit_file ) ;
	rewind( bit_file ) ;
	cout << "0%                                  100%" << endl ;
	for (prog=prev_prog=0;;) {
		for ( int i = 0 ; i < 8; i++, cur <<= 1 )  
			frame[i] = (cur & 0x80 ) != 0 ;

		size+=1 ;
		prog = size*40/total ;
		if ( prog != prev_prog ) {
			cout << "=" ;
			cout.flush() ;
		}
		prev_prog = prog ;
	
 		if ( fread( &next, 1, 1, bit_file ) == 0 ) 
			break ;
		dev.setDR( frame, dummy, 8 ) ;
		cur = next ;
	}
	cout << endl ;
	
	dev.setDR( frame, dummy, 7 ) ;
	dev.exitDR( frame[7] ) ;
	fclose( bit_file ) ;

/*....Start FPGA.....*/
	dev.selectIR() ;
	if ( prom_present ) dev.setIR( BYPASS_PROM, 8 ) ;
	dev.setIR( JSTART, 5 ) ;
	dev.exitIR( JSTART[5] ) ;

	dev.selectRunTestIdle() ;
	for ( int i = 0 ; i < 20 ; i++ )		/* Start up sequence */
		dev.execute( 0, 0 ) ;
exit:
	dev.close() ;

	return 0 ;
}
